==> ocre_v0_034.tgz: ocre-0.034/dcsw/treesdcsw.c <==

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "treesdcsw.h"
#include "tabledcsw.h"
#include "dcsw.h"
/*
#include "errors.h"
*/

int FALSE=0;
int TRUE=1;

// -------------------- lee --------------------

void leeArbolesN (tplarboles * nllarboles, char * dir)
{
   char nf[128];
   char nfa[128];
   FILE * fd;
   int nl, l;
   int na, ia, na_enfich;
   tplarboles plarbs, plarbsAnt;
   char nom[128];
   int nc;
   sprintf(nf, "%s/00index", dir);
   fd = fopen (nf, "r");
   //  compruebaFd (fd, nf, "imagio.leeImagen");
   if (fd == NULL) {
      printf ("treesdcsw: error at leeArboles opening index %s\n",nf);
      exit (1);
      }
   fscanf (fd, "%d", &nl);
   for (l=0; l<nl; l++) {
      fscanf (fd, "%d", &na);
      plarbs = (tplarboles) malloc (sizeof(struct tlarboles));
      plarbs->num = na;
      plarbs->larb = (tpnodo *) malloc(na * sizeof(tpnodo));
      fscanf(fd, "%s", nom);
      nc = strlen (nom);
      plarbs->nombre = (char *) malloc((nc+1) * sizeof(char));
      strcpy (plarbs->nombre, nom); 
      if (0) {                               // codigo antiguo
         for (ia=0; ia<na; ia++) {
            sprintf(nfa, "%s/%s=%02d.dcw", dir, nom, ia);
            plarbs->larb[ia] = leeArbol(nfa);
            }
         }
      else {
         sprintf(nfa, "%s/%s.dcw", dir, nom);
         leeArbol2 (nfa, plarbs->larb, &na_enfich);
         if (na_enfich < na)
            plarbs->num = na_enfich ;
         }
      if (l == 0)
         * nllarboles = plarbs ;
      if (l > 0)
         plarbsAnt->next = plarbs;
      plarbsAnt = plarbs;
      }
   plarbsAnt->next = (tplarboles) 0L;
   fclose (fd);
}

void leeArboles (char * dir)
{
   leeArbolesN (&llarboles, dir);
}

// -------------------- escribe --------------------

void escribeArboles0 (char * dir, char * nomc, int nar)
{
   int ia;
   char nomFich[128];
   tpnodo arbol;
   borraMascara ();
   for (ia=0; ia<nar; ia++) {
      sprintf (nomFich, "%s/arbol_%s=%02d.dcw", dir, nomc, ia);
      escribeArbol (nomFich, arbol = arbolId3() );
      actualizaMascara ();
      }
}

void escribeArboles (char * dir, char * nomc, int nar)
{
   fijaFunc (nomc);
   escribeArboles0 (dir, nomc, nar);
}

// escribe ... calculados en esta funcion  (arbolId3)
void escribeArboles1 (char * dir, char * nomc, int nar)
{
   int ia;
   char nomFich[128];
   tpnodo arbol[40];
   tpnodo Arbol;
   fijaFunc (nomc);
   borraMascara ();
   for (ia=0; ia<nar; ia++) {
      arbol[ia] = Arbol = arbolId3();
      actualizaMascara ();
      }
   // sprintf (nomFich, "%s/arbol-%s=02d.dcw", dir, nomc, nar);
   sprintf (nomFich, "%s/arbol-%s.dcw", dir, nomc);
   escribeArbol2 (nomFich, arbol, nar);
}

// en directorio dir,
// nnomc (numero de) nombres de caracter nomc[i],
// separados por cs
// nar (numero de) arboles
void escribeArbolesO (char * dir, char ** nomc, int nnomc, char *cs, int nar)
{
   int i;
   char nomcc[128];
   fijaFunc (nomc[0]);
   strncpy (nomcc, cs, 2);
   strncat (nomcc, nomc[0], 20);
   for (i=1; i<nnomc; i++){
      oFunc (nomc[i]);
      strncat (nomcc, cs, 2);
      strncat (nomcc, nomc[i], 20);
      }
   escribeArboles0 (dir, nomcc, nar);
}

// -------------------- evalua --------------------

void evaluaConArbolesInd (tplarboles plarbs, int v[], int res[])
         // res[]   int .. [3]
{
   int ia;
   int r;
   res [nd_falso]  = 0;
   res [nd_cierto] = 0;
   res [nd_nose ]  = 0;
   for (ia=0; ia < plarbs->num; ia++){
      r = evalArbolVector (plarbs->larb[ia], v);
      res [r] ++ ;
      }
}

tplarboles buscaNomEnArbolesN (tplarboles nllarboles, char * nomCar)
{
   tplarboles plarbs;
   for(plarbs = nllarboles; plarbs != 0L; plarbs = plarbs->next){
      if (strcmp(nomCar,plarbs->nombre) == 0) break ;
      }
   if (plarbs == 0L) {
      printf ("treesdcsw: error en evaluaConArbolesN buscando nomCar=%s\n", nomCar);
      exit (1);
      }
   return plarbs;
}

tplarboles buscaNomEnArboles (char * nomCar)
{
   tplarboles plarbs;
   for(plarbs = llarboles; plarbs != 0L; plarbs = plarbs->next){
      if (strcmp(nomCar,plarbs->nombre) == 0) break ;
      }
   if (plarbs == 0L) {
      printf ("treesdcsw: error en evaluaConArboles buscando nomCar=%s\n", nomCar);
      exit (1);
      }
   return plarbs;
}

void evaluaConArbolesN (tplarboles nllarboles, char * nomCar, int v[], int res[])
         // res[]   int .. [3]
{
   tplarboles plarbs;         
   plarbs = buscaNomEnArbolesN (nllarboles, nomCar);
   evaluaConArbolesInd (plarbs, v, res);
   // printf ("evaluaConArbolesN %s %2d %2d %2d\n",plarbs, res[0], res[1], res[2]);
}

void evaluaConArboles (char * nomCar, int v[], int res[])
         /* res[]   int .. [3] */
{
   tplarboles plarbs;         
   plarbs = buscaNomEnArboles (nomCar);
   evaluaConArbolesInd (plarbs, v, res);
   // printf ("evaluaConArboles %s %2d %2d %2d\n",plarbs, res[0], res[1], res[2]);
}

float valProp (int * res)
{
   int narb;
   narb = res[0]+res[1]+res[2] ;
   return (0.5 + res[1]) / (narb+1.0) ;
}
 //
float umbrValPropDef = 0.45;  //   0.45  (o 0.25?)

void reordenaPropuestas (tppropues lprop, char * nom, int * res, int npp, int * pnprp)
{
   int i, j;
   int encontrado;
   // busca posicion
   encontrado = FALSE;
   i = 0;
   while (!encontrado && (i < *pnprp)) {
      if (valProp(res) > valProp(lprop[i].res)) encontrado = TRUE;
      else i++;
      }
   // la posicion a insertar es  i 
   if (i == npp) return ;    // no cabe, no es de las npp mejores
   if (i < *pnprp) {
      // deplazar lo que haya tras  i (inclusive), i .. (*pnprp)-1
      for (j = (*pnprp)-1; j>=i; j--) {
         lprop[j+1].nombre = lprop[j].nombre ;
         lprop[j+1].narb   = lprop[j].narb   ;
         lprop[j+1].res[0] = lprop[j].res[0] ;
         lprop[j+1].res[1] = lprop[j].res[1] ;
         lprop[j+1].res[2] = lprop[j].res[2] ;
         }
      }
   // introducir la propuesta en  i 
   lprop[i].nombre = nom ;
   lprop[i].narb   = res[0]+res[1]+res[2] ;
   lprop[i].res[0] = res[0] ;
   lprop[i].res[1] = res[1] ;
   lprop[i].res[2] = res[2] ;
   //
   (*pnprp)++ ;
   // printf ("=====%4d %4d %4d %s\n", i, npp, *pnprp, nom);
}

void clasificaConArbolesN (tplarboles llarboles,
                                        int v[], tppropues lprop, int npp, int * pnprp,
			   float umbrValProp)
{
   int res[3];
   tplarboles plarbs;
   * pnprp = 0;
   for(plarbs = llarboles; plarbs != 0L; plarbs = plarbs->next){
      evaluaConArbolesInd (plarbs, v, res);
// para ver traza de dcsw
//    if (res[0]!=5)
//       printf("%1d %1d %1d %10s\n", res[0], res[1], res[2], plarbs->nombre);
//
      if (valProp (res) > umbrValProp)
         reordenaPropuestas (lprop, plarbs->nombre, res, npp, pnprp);
      }
// para ver traza de dcsw
//  printf ("\n");
//
}

void clasificaConArboles (int v[], tppropues lprop, int npp, int * pnprp)
{
   clasificaConArbolesN (llarboles, v, lprop, npp, pnprp, umbrValPropDef);
}

==> ocre_v0_042.tgz: ocre-0.042/dcsw/treesdcsw.c <==

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "treesdcsw.h"
#include "tabledcsw.h"
#include "dcsw.h"
/*
#include "errors.h"
*/

int FALSE=0;
int TRUE=1;

// -------------------- lee --------------------

void leeArbolesN (tplarboles * nllarboles, char * dir)
{
   char nf[128];
   char nfa[128];
   FILE * fd;
   int nl, l;
   int na, ia, na_enfich;
   tplarboles plarbs, plarbsAnt;
   char nom[128];
   int nc;
   sprintf(nf, "%s/00index", dir);
   fd = fopen (nf, "r");
   //  compruebaFd (fd, nf, "imagio.leeImagen");
   if (fd == NULL) {
      printf ("treesdcsw: error at leeArboles opening index %s\n",nf);
      exit (1);
      }
   fscanf (fd, "%d", &nl);
   for (l=0; l<nl; l++) {
      fscanf (fd, "%d", &na);
      plarbs = (tplarboles) malloc (sizeof(struct tlarboles));
      plarbs->num = na;
      plarbs->larb = (tpnodo *) malloc(na * sizeof(tpnodo));
      fscanf(fd, "%s", nom);
      nc = strlen (nom);
      plarbs->nombre = (char *) malloc((nc+1) * sizeof(char));
      strcpy (plarbs->nombre, nom); 
      if (0) {                               // codigo antiguo
         for (ia=0; ia<na; ia++) {
            sprintf(nfa, "%s/%s=%02d.dcw", dir, nom, ia);
            plarbs->larb[ia] = leeArbol(nfa);
            }
         }
      else {
         sprintf(nfa, "%s/%s.dcw", dir, nom);
         leeArbol2 (nfa, plarbs->larb, &na_enfich);
         if (na_enfich < na)
            plarbs->num = na_enfich ;
         }
      if (l == 0)
         * nllarboles = plarbs ;
      if (l > 0)
         plarbsAnt->next = plarbs;
      plarbsAnt = plarbs;
      }
   plarbsAnt->next = (tplarboles) 0L;
   fclose (fd);
}

void leeArboles (char * dir)
{
   leeArbolesN (&llarboles, dir);
}

// -------------------- escribe --------------------

void escribeArboles0 (char * dir, char * nomc, int nar)
{
   int ia;
   char nomFich[128];
   tpnodo arbol;
   borraMascara ();
   for (ia=0; ia<nar; ia++) {
      sprintf (nomFich, "%s/arbol_%s=%02d.dcw", dir, nomc, ia);
      escribeArbol (nomFich, arbol = arbolId3() );
      actualizaMascara ();
      }
}

void escribeArboles (char * dir, char * nomc, int nar)
{
   fijaFunc (nomc);
   escribeArboles0 (dir, nomc, nar);
}

// escribe ... calculados en esta funcion  (arbolId3).., y borra
void escribeArboles1 (char * dir, char * nomc, int nar)
{
   int ia;
   char nomFich[128];
   tpnodo arbol[40];
   tpnodo Arbol;
   fijaFunc (nomc);
   borraMascara ();
   for (ia=0; ia<nar; ia++) {
      arbol[ia] = Arbol = arbolId3();
      actualizaMascara ();
      }
   // sprintf (nomFich, "%s/arbol-%s=02d.dcw", dir, nomc, nar);
   sprintf (nomFich, "%s/arbol-%s.dcw", dir, nomc);
   escribeArbol2 (nomFich, arbol, nar);
   for (ia=0; ia<nar; ia++) {
      destruyeArbol (arbol[ia]);
      }
}

// en directorio dir,
// nnomc (numero de) nombres de caracter nomc[i],
// separados por cs
// nar (numero de) arboles
void escribeArbolesO (char * dir, char ** nomc, int nnomc, char *cs, int nar)
{
   int i;
   char nomcc[128];
   fijaFunc (nomc[0]);
   strncpy (nomcc, cs, 2);
   strncat (nomcc, nomc[0], 20);
   for (i=1; i<nnomc; i++){
      oFunc (nomc[i]);
      strncat (nomcc, cs, 2);
      strncat (nomcc, nomc[i], 20);
      }
   escribeArboles0 (dir, nomcc, nar);
}

// -------------------- evalua --------------------

void evaluaConArbolesInd (tplarboles plarbs, int v[], int res[])
         // res[]   int .. [3]
{
   int ia;
   int r;
   res [nd_falso]  = 0;
   res [nd_cierto] = 0;
   res [nd_nose ]  = 0;
   for (ia=0; ia < plarbs->num; ia++){
      r = evalArbolVector (plarbs->larb[ia], v);
      res [r] ++ ;
      }
}

tplarboles buscaNomEnArbolesN (tplarboles nllarboles, char * nomCar)
{
   tplarboles plarbs;
   for(plarbs = nllarboles; plarbs != 0L; plarbs = plarbs->next){
      if (strcmp(nomCar,plarbs->nombre) == 0) break ;
      }
   if (plarbs == 0L) {
      printf ("treesdcsw: error en evaluaConArbolesN buscando nomCar=%s\n", nomCar);
      exit (1);
      }
   return plarbs;
}

tplarboles buscaNomEnArboles (char * nomCar)
{
   tplarboles plarbs;
   for(plarbs = llarboles; plarbs != 0L; plarbs = plarbs->next){
      if (strcmp(nomCar,plarbs->nombre) == 0) break ;
      }
   if (plarbs == 0L) {
      printf ("treesdcsw: error en evaluaConArboles buscando nomCar=%s\n", nomCar);
      exit (1);
      }
   return plarbs;
}

void evaluaConArbolesN (tplarboles nllarboles, char * nomCar, int v[], int res[])
         // res[]   int .. [3]
{
   tplarboles plarbs;         
   plarbs = buscaNomEnArbolesN (nllarboles, nomCar);
   evaluaConArbolesInd (plarbs, v, res);
   // printf ("evaluaConArbolesN %s %2d %2d %2d\n",plarbs, res[0], res[1], res[2]);
}

void evaluaConArboles (char * nomCar, int v[], int res[])
         /* res[]   int .. [3] */
{
   tplarboles plarbs;         
   plarbs = buscaNomEnArboles (nomCar);
   evaluaConArbolesInd (plarbs, v, res);
   // printf ("evaluaConArboles %s %2d %2d %2d\n",plarbs, res[0], res[1], res[2]);
}

float valProp (int * res)
{
   int narb;
   narb = res[0]+res[1]+res[2] ;
   return (0.5 + res[1]) / (narb+1.0) ;
}
 //
float umbrValPropDef = 0.45;  //   0.45  (o 0.25?)

void reordenaPropuestas (tppropues lprop, char * nom, int * res, int npp, int * pnprp)
{
   int i, j;
   int encontrado;
   // busca posicion
   encontrado = FALSE;
   i = 0;
   while (!encontrado && (i < *pnprp)) {
      if (valProp(res) > valProp(lprop[i].res)) encontrado = TRUE;
      else i++;
      }
   // la posicion a insertar es  i 
   if (i == npp) return ;    // no cabe, no es de las npp mejores
   if (i < *pnprp) {
      // deplazar lo que haya tras  i (inclusive), i .. (*pnprp)-1
      for (j = (*pnprp)-1; j>=i; j--) {
         lprop[j+1].nombre = lprop[j].nombre ;
         lprop[j+1].narb   = lprop[j].narb   ;
         lprop[j+1].res[0] = lprop[j].res[0] ;
         lprop[j+1].res[1] = lprop[j].res[1] ;
         lprop[j+1].res[2] = lprop[j].res[2] ;
         }
      }
   // introducir la propuesta en  i 
   lprop[i].nombre = nom ;
   lprop[i].narb   = res[0]+res[1]+res[2] ;
   lprop[i].res[0] = res[0] ;
   lprop[i].res[1] = res[1] ;
   lprop[i].res[2] = res[2] ;
   //
   (*pnprp)++ ;
   // printf ("=====%4d %4d %4d %s\n", i, npp, *pnprp, nom);
}

void apuntaPropuestasNonulas (tppropues lprop, char * nom, int * res, int npp, int * pnprp)
{
   int i, j;
   int encontrado;
   // busca posicion
   encontrado = FALSE;
   i = *pnprp;
   // la posicion a insertar es  i 
   if (i == npp) return ;    // no cabe, no es de las npp mejores
   // introducir la propuesta en  i 
   lprop[i].nombre = nom ;
   lprop[i].narb   = res[0]+res[1]+res[2] ;
   lprop[i].res[0] = res[0] ;
   lprop[i].res[1] = res[1] ;
   lprop[i].res[2] = res[2] ;
   //
   (*pnprp)++ ;
   // printf ("=====%4d %4d %4d %s\n", i, npp, *pnprp, nom);
}

void clasificaConArbolesN (tplarboles llarboles,
                                        int v[], tppropues lprop, int npp, int * pnprp,
			   float umbrValProp)
{
   int res[3];
   tplarboles plarbs;
   * pnprp = 0;
   for(plarbs = llarboles; plarbs != 0L; plarbs = plarbs->next){
      evaluaConArbolesInd (plarbs, v, res);
// para ver traza de dcsw
//    if (res[0]!=5)
//       printf("%1d %1d %1d %10s\n", res[0], res[1], res[2], plarbs->nombre);
//
      if (valProp (res) > umbrValProp)
         reordenaPropuestas (lprop, plarbs->nombre, res, npp, pnprp);
      }
// para ver traza de dcsw
//  printf ("\n");
//
}

void clasificaConArboles (int v[], tppropues lprop, int npp, int * pnprp)
{
   clasificaConArbolesN (llarboles, v, lprop, npp, pnprp, umbrValPropDef);
}

void informaConArbolesN (tplarboles llarboles,
                                        int v[], tppropues lprop, int npp, int * pnprp)
{
   int res[3];
   tplarboles plarbs;
   * pnprp = 0;
   for(plarbs = llarboles; plarbs != 0L; plarbs = plarbs->next){
      evaluaConArbolesInd (plarbs, v, res);
// para ver traza de dcsw
//    if (res[0]!=5)
//       printf("%1d %1d %1d %10s\n", res[0], res[1], res[2], plarbs->nombre);
//
      if (res[1] > 0)
         apuntaPropuestasNonulas (lprop, plarbs->nombre, res, npp, pnprp);
      }
// para ver traza de dcsw
//  printf ("\n");
//
}

void informaConArboles (int v[], tppropues lprop, int npp, int * pnprp)
{
   informaConArbolesN (llarboles, v, lprop, npp, pnprp);
}
